package com.gic.haoban.manage.web.controller;

import cn.hutool.core.collection.CollectionUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.gic.api.base.commons.Page;
import com.gic.commons.util.DateUtil;
import com.gic.enterprise.api.dto.EnterpriseDTO;
import com.gic.enterprise.api.dto.*;
import com.gic.enterprise.api.service.AuthorizeService;
import com.gic.enterprise.api.service.EnterpriseService;
import com.gic.enterprise.api.service.StoreService;
import com.gic.haoban.base.api.common.BasePageInfo;
import com.gic.haoban.base.api.common.ServiceResponse;
import com.gic.haoban.common.utils.EntityUtil;
import com.gic.haoban.common.utils.HaobanResponse;
import com.gic.haoban.manage.api.dto.DepartmentDTO;
import com.gic.haoban.manage.api.dto.*;
import com.gic.haoban.manage.api.enums.ChannelCodeEnum;
import com.gic.haoban.manage.api.enums.SecretTypeEnum;
import com.gic.haoban.manage.api.service.*;
import com.gic.haoban.manage.web.anno.IgnoreLogin;
import com.gic.haoban.manage.web.auth.AuthRequestUtil;
import com.gic.haoban.manage.web.config.Config;
import com.gic.haoban.manage.web.errCode.HaoBanErrCode;
import com.gic.haoban.manage.web.qo.MiniprogramSettingQo;
import com.gic.haoban.manage.web.qo.WxEnterpriseBindQo;
import com.gic.haoban.manage.web.vo.EnterpriseDetailVO;
import com.gic.haoban.manage.web.vo.EnterpriseSearchVO;
import com.gic.haoban.manage.web.vo.SecretSettingVO;
import com.gic.haoban.manage.web.vo.StoreVo;
import com.gic.quartz.api.dto.QuartzTaskDTO;
import com.gic.redis.data.util.RedisUtil;
import com.gic.wechat.api.dto.qywx.UserDTO;
import com.gic.wechat.api.service.qywx.QywxUserApiService;
import com.gic.wechat.token.api.dto.QywxCorpInfoDTO;
import com.gic.wechat.token.api.service.QywxTokenManageService;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.*;
import java.util.stream.Collectors;

@RestController
public class WxEnterpriseController extends WebBaseController {
    private static final Logger logger = LoggerFactory.getLogger(WxEnterpriseController.class);
    @Autowired
    private WxEnterpriseRelatedApiService wxEnterpriseRelatedApiService;
    @Autowired
    private EnterpriseService enterpriseService;
    @Autowired
    private WxEnterpriseApiService wxEnterpriseApiService;
    @Autowired
    private DepartmentApiService departmentApiService;
    @Autowired
    private StoreService storeService;
    @Autowired
    private QywxTokenManageService qywxTokenManageService;
    @Autowired
    private Config config;
    @Autowired
    private QywxUserApiService qywxUserApiService;
    @Autowired
    private StaffApiService staffApiService;
    @Autowired
    private DealSyncOperationApiService dealSyncOperationApiService;
    @Autowired
    private MaterialApiService materialApiService;
    @Autowired
    private AuthorizeService authorizeService;


    //授权企业列表
    @RequestMapping("wxa-enterprise-list")
    public HaobanResponse wxEnterpriseList() {
        LoginDTO login = (LoginDTO) AuthRequestUtil.getLoginUser();
        String wxEnterpriseId = login.getWxEnterpriseId();
        List<EnterpriseDetailDTO> list = wxEnterpriseRelatedApiService.listEnterpriseByWxEnterpriseId(wxEnterpriseId);
        List<EnterpriseDetailVO> detailVOS = EntityUtil.changeEntityListByOrika(EnterpriseDetailVO.class, list);
        if (CollectionUtil.isNotEmpty(detailVOS)) {
            List<String> enterpriseIds = detailVOS.stream().map(s -> s.getEnterpriseId()).collect(Collectors.toList());
            List<EnterpriseDTO> enterpriseList = enterpriseService.listEnterpriseByIds(enterpriseIds);
            Map<String, EnterpriseDTO> map = com.gic.commons.util.CollectionUtil.toMap(enterpriseList, "enterpriseId");
            for (EnterpriseDetailVO dto : detailVOS) {
                List<EnterpriseWxaRelationDTO> relationDTOS = authorizeService.queryList(dto.getEnterpriseId());
                EnterpriseDTO enterpriseDTO = map.get(dto.getEnterpriseId());
                EnterpriseWxaRelationDTO wxaRelationDTO = relationDTOS.stream().filter(mid -> mid.getWxaAppkey().equals(enterpriseDTO.getMallMiniprogramAppid())).findFirst().orElse(null);
                if (wxaRelationDTO != null) {
                    dto.setEnterpriseName(enterpriseDTO.getEnterpriseName());
                    dto.setImageUrl(wxaRelationDTO.getAuthImg());
                    dto.setAppId(wxaRelationDTO.getWxaAppkey());
                    dto.setMiniprogramName(wxaRelationDTO.getAuthName());
                    dto.setTitle(wxaRelationDTO.getAuthName());
                }
            }
        }
        return resultResponse(HaoBanErrCode.ERR_1, detailVOS);
    }


    //授权企业列表
    @RequestMapping("wx-enterprise-list")
    public HaobanResponse wxaEnterpriseList() {
        LoginDTO login = (LoginDTO) AuthRequestUtil.getLoginUser();
        String wxEnterpriseId = login.getWxEnterpriseId();
        List<EnterpriseDetailDTO> list = wxEnterpriseRelatedApiService.listEnterpriseByWxEnterpriseId(wxEnterpriseId);
        if (CollectionUtil.isNotEmpty(list)) {
            List<String> enterpriseIds = list.stream().map(s -> s.getEnterpriseId()).collect(Collectors.toList());
            List<EnterpriseDTO> enterpriseList = enterpriseService.listEnterpriseByIds(enterpriseIds);
            Map<String, EnterpriseDTO> map = com.gic.commons.util.CollectionUtil.toMap(enterpriseList, "enterpriseId");
            for (EnterpriseDetailDTO dto : list) {
                EnterpriseDTO enterpriseDTO = map.get(dto.getEnterpriseId());

                if (enterpriseDTO != null) {
                    dto.setEnterpriseName(enterpriseDTO.getEnterpriseName());
                    dto.setEnterpriseLogo(enterpriseDTO.getQcloudImageUrl());
                    dto.setBrandName(enterpriseDTO.getBrandName());
                }
            }
        }
        return resultResponse(HaoBanErrCode.ERR_1, list);

    }

    //企业信息
    @IgnoreLogin
    @RequestMapping("wxEnterprise-detail")
    public HaobanResponse enterpriseDetail(String wxEnterpriseId) {
        if (StringUtils.isEmpty(wxEnterpriseId)) {
            return resultResponse(HaoBanErrCode.ERR_1);
        }
        WxEnterpriseDTO wxEnterpriseDTO = wxEnterpriseApiService.getOne(wxEnterpriseId);
        if (wxEnterpriseDTO == null) {
            return resultResponse(HaoBanErrCode.ERR_100018);
        }
        wxEnterpriseDTO.setSmallVersion("免费版");
        wxEnterpriseDTO.setStoreTotal(departmentApiService.totalStoreCountByEnterpriseId(wxEnterpriseId));
        return resultResponse(HaoBanErrCode.ERR_1, wxEnterpriseDTO);
    }

    //企业信息
    @IgnoreLogin
    @RequestMapping("wxEnterprise-refresh")
    public HaobanResponse enterpriseRefresh(String wxEnterpriseId) {
        if (StringUtils.isEmpty(wxEnterpriseId)) {
            return resultResponse(HaoBanErrCode.ERR_1);
        }
        WxEnterpriseDTO wxEnterpriseDTO = wxEnterpriseApiService.getOne(wxEnterpriseId);
        if (wxEnterpriseDTO == null) {
            return resultResponse(HaoBanErrCode.ERR_10012);
        }
        QywxCorpInfoDTO dto = qywxTokenManageService.getCorpInfo(wxEnterpriseDTO.getCorpid(), config.getWxSuiteid());
        if (dto == null) {
            return resultResponse(HaoBanErrCode.ERR_10012);
        }
        Long verifiedEndTime = dto.getVerifiedEndTime();
        logger.info("【企业同步】dto={}", JSON.toJSONString(dto));
        WxEnterpriseDTO wxDTO = EntityUtil.changeEntityByJSON(WxEnterpriseDTO.class, dto);
        if (verifiedEndTime != null) {
            wxDTO.setVerifiedEndTime(new Date(verifiedEndTime * 1000));
        }
        wxDTO.setWxEnterpriseId(wxEnterpriseId);
        Integer contactFlag = wxEnterpriseDTO.getContactFlag();
        if (contactFlag == 0) {
            wxDTO.setContactFlag(1);
        }
        wxEnterpriseApiService.update(wxDTO);
        return resultResponse(HaoBanErrCode.ERR_1, wxEnterpriseDTO);
    }

    //gic企业列表
    @IgnoreLogin
    @RequestMapping("enterprise-search")
    public HaobanResponse gicEnterpriseDetail(String enterpriseName) {
        if (StringUtils.isEmpty(enterpriseName)) {
            return resultResponse(HaoBanErrCode.ERR_1);
        }
        EnterpriseAndUserDTO dto = enterpriseService.getEnterpriseAndUserByName(enterpriseName);
        EnterpriseSearchVO vo = EntityUtil.changeEntityByJSON(EnterpriseSearchVO.class, dto);
        if (vo != null) {
            String enterpriseId = vo.getEnterpriseId();
            EnterpriseDTO enterprise = enterpriseService.getEnterpriseById(enterpriseId);
            if (enterprise != null) {
                vo.setBrandName(enterprise.getBrandName());
            }
        }
        return resultResponse(HaoBanErrCode.ERR_1, vo);
    }

    //微信企业绑定接口
    @RequestMapping("wx-enterprise-bind")
    public HaobanResponse wxEnterpriseBind(WxEnterpriseBindQo bindQo) {
        if (StringUtils.isAnyBlank(bindQo.getEnterpriseId(), bindQo.getVersion())) {
            return resultResponse(HaoBanErrCode.ERR_2);
        }
        LoginDTO login = (LoginDTO) AuthRequestUtil.getLoginUser();
        String wxEnterpriseId = login.getWxEnterpriseId();
        StaffDTO staff = login.getStaffDTO();
        String staffId = staff.getStaffId();
        String staffName = staff.getStaffName();

        WxEnterpriseRelationDetailDTO detailDTO = new WxEnterpriseRelationDetailDTO();
        detailDTO.setWxEnterpriseRelatedId(bindQo.getWxEnterpriseRelatedId());
        detailDTO.setEnterpriseId(bindQo.getEnterpriseId());
        detailDTO.setWxEnterpriseId(wxEnterpriseId);
        detailDTO.setStaffId(staffId);
        detailDTO.setStaffName(staffName);
        detailDTO.setMemberOpenCardFlag(bindQo.getMemberOpenCardFlag());
        detailDTO.setWxEnterpriseType(bindQo.getWxEnterpriseType());
        List<StoreOrGroupInfoDTO> storeOrGroupInfoDTOS = JSONObject.parseArray(bindQo.getRelations(), StoreOrGroupInfoDTO.class);
        detailDTO.setRelations(storeOrGroupInfoDTOS);
        ServiceResponse resp = wxEnterpriseRelatedApiService.wxEnterpriseBind(detailDTO);
        if (resp.getCode() != 1) {
            HaoBanErrCode.ERR_DEFINE.setMsg(resp.getMessage());
            return resultResponse(HaoBanErrCode.ERR_DEFINE);
        }
        return resultResponse(HaoBanErrCode.ERR_1);

    }

    //溢出门店列表
    @RequestMapping("store-full-list")
    public HaobanResponse storeFullList(BasePageInfo basePageInfo, String keyWord) {
        LoginDTO login = (LoginDTO) AuthRequestUtil.getLoginUser();
        String wxEnterpriseId = login.getWxEnterpriseId();
        int maxVersionCount = 0;
        List<EnterpriseDetailDTO> enterpriseList = wxEnterpriseRelatedApiService.listEnterpriseByWxEnterpriseId(wxEnterpriseId);
        if (CollectionUtil.isEmpty(enterpriseList)) {
            return resultResponse(HaoBanErrCode.ERR_10010);
        }
        List<String> enterpriseIds = enterpriseList.stream().map(s -> s.getEnterpriseId()).collect(Collectors.toList());
        //获取门店列表
        Page page1 = new Page<>();
        page1.setCurrentPage(1);
        page1.setPageSize(Integer.MAX_VALUE);
        StoreSearchDTO storeSearchDTO = new StoreSearchDTO();
        storeSearchDTO.setEnterpriseIdList(enterpriseIds);
        storeSearchDTO.setSearchName(keyWord);
        Page storePage = storeService.storeListPage(page1, storeSearchDTO);
        if (storePage == null || storePage.getResult() == null) {
            return resultResponse(HaoBanErrCode.ERR_10009);
        }
        List<StoreDTO> t_list = storePage.getResult();
        List<String> storeIds = t_list.stream().map(s -> s.getStoreId()).collect(Collectors.toList());
        if (StringUtils.isEmpty(keyWord)) {
            storeIds = null;
        }
        Page<DepartmentDTO> page = departmentApiService.pageFullStoreByWxEnterpriseId(wxEnterpriseId, keyWord, storeIds, maxVersionCount, basePageInfo);
        List<DepartmentDTO> list = page.getResult();
        List<StoreVo> resultList = EntityUtil.changeEntityListByJSON(StoreVo.class, list);
        List<String> t_storeIds = list.stream().map(s -> s.getRelatedId()).collect(Collectors.toList());
        String[] storeIdArr = t_storeIds.toArray(new String[t_storeIds.size()]);
        List<StoreDTO> storeList = storeService.getStores(storeIdArr);
        Map<String, StoreDTO> map = com.gic.commons.util.CollectionUtil.toMap(storeList, "storeId");

        for (StoreVo vo : resultList) {
            if (map.get(vo.getRelatedId()) != null) {
                vo.setStoreCode(map.get(vo.getRelatedId()).getStoreCode());
                vo.setStoreId(map.get(vo.getRelatedId()).getStoreId());
                vo.setStoreName(map.get(vo.getRelatedId()).getStoreName());
            }
        }

        Page resultPage = new Page<>();
        resultPage.setCurrentPage(page.getCurrentPage());
        resultPage.setPageSize(page.getPageSize());
        resultPage.setTotalCount(page.getTotalCount());
        resultPage.setResult(resultList);
        return resultResponse(HaoBanErrCode.ERR_1, resultPage);

    }

    //微信企业是否设置key
    @RequestMapping("is-wx-enterprise-secret-set")
    public HaobanResponse isWxEnterpriseSecretSet() {
        String wxEnterpriseId = "";
        LoginDTO login = (LoginDTO) AuthRequestUtil.getLoginUser();
        wxEnterpriseId = login.getWxEnterpriseId();
        List<SecretSettingDTO> listSecret = wxEnterpriseApiService.listSecret(wxEnterpriseId);
        if (CollectionUtils.isEmpty(listSecret)) {
            return resultResponse(HaoBanErrCode.ERR_1, false);
        }
        Set<Integer> secretTypeList = listSecret.stream().filter(secretSettingDTO -> secretSettingDTO.getCheckFlag() == 1).map(secretSettingDTO -> secretSettingDTO.getSecretType()).collect(Collectors.toSet());
        if (CollectionUtils.isEmpty(secretTypeList)) {
            return resultResponse(HaoBanErrCode.ERR_1, false);
        }
        if ((secretTypeList.contains(SecretTypeEnum.HAOBAN_HELP.getVal()) || secretTypeList.contains(SecretTypeEnum.CUSTOMIZED_APP.getVal()))
                && secretTypeList.contains(SecretTypeEnum.CONTACT_CUSTOMER.getVal())) {
            return resultResponse(HaoBanErrCode.ERR_1, true);
        } else {
            return resultResponse(HaoBanErrCode.ERR_1, false);
        }
    }

    //微信企业绑定接口绑定通讯录key
    @RequestMapping("wx-enterprise-contact-secret-set")
    public HaobanResponse wxEnterpriseSecretSet(String contactSecret) {
        if (StringUtils.isBlank(contactSecret)) {
            return resultResponse(HaoBanErrCode.ERR_2);
        }
        String wxEnterpriseId = "";
        LoginDTO login = (LoginDTO) AuthRequestUtil.getLoginUser();
        wxEnterpriseId = login.getWxEnterpriseId();
        WxEnterpriseDTO wxEnterprise = wxEnterpriseApiService.getOne(wxEnterpriseId);
        if (wxEnterprise == null) {
            return resultResponse(HaoBanErrCode.ERR_10012);
        }
        contactSecret = contactSecret.trim();
        UserDTO user = qywxUserApiService.getSelfWorkWxUser(wxEnterprise.getCorpid(), contactSecret, getWxUserId(wxEnterpriseId, login.getStaffDTO()));
        logger.info("【保存联系人key】user={}", JSON.toJSONString(user));
        SecretSettingDTO secretSetting = wxEnterpriseApiService.getSecretSetting(wxEnterpriseId, SecretTypeEnum.CONTACT_SECRET.getVal(), null);
        if (null == secretSetting) {
            secretSetting = new SecretSettingDTO();
        }
        secretSetting.setSecretType(SecretTypeEnum.CONTACT_SECRET.getVal());
        secretSetting.setSecretName(SecretTypeEnum.CONTACT_SECRET.getName());
        secretSetting.setWxEnterpriseId(wxEnterpriseId);
        secretSetting.setEnterpriseId("-1");
        if (user != null) {
            secretSetting.setSecretVal(contactSecret);
            secretSetting.setCheckFlag(1);
            wxEnterpriseApiService.saveSecretSetting(secretSetting);
            return resultResponse(HaoBanErrCode.ERR_1, true);
        } else {
            secretSetting.setCheckFlag(0);
            wxEnterpriseApiService.saveSecretSetting(secretSetting);
            return resultResponse(HaoBanErrCode.ERR_10013);
        }

    }

    //微信企业绑定接口绑定通讯录key
    @RequestMapping("wx-enterprise-wx-secret-set")
    public HaobanResponse wxEnterpriseWxSecretSet(String wxSecretKey) {
        if (StringUtils.isAnyBlank(wxSecretKey)) {
            return resultResponse(HaoBanErrCode.ERR_2);
        }
        String wxEnterpriseId = "";
        LoginDTO login = (LoginDTO) AuthRequestUtil.getLoginUser();
        wxEnterpriseId = login.getWxEnterpriseId();
        WxEnterpriseDTO wxEnterprise = wxEnterpriseApiService.getOne(wxEnterpriseId);
        if (wxEnterprise == null) {
            return resultResponse(HaoBanErrCode.ERR_10012);
        }
        wxSecretKey = wxSecretKey.trim();
        String res = qywxUserApiService.listSelfExternalUserid(wxEnterprise.getCorpid(), wxSecretKey, getWxUserId(wxEnterpriseId, login.getStaffDTO()));
        logger.info("【保存联系人key】res={}", res);

        SecretSettingDTO secretSetting = wxEnterpriseApiService.getSecretSetting(wxEnterpriseId, SecretTypeEnum.CONTACT_CUSTOMER.getVal(), null);
        if (null == secretSetting) {
            secretSetting = new SecretSettingDTO();
        }
        secretSetting.setSecretType(SecretTypeEnum.CONTACT_CUSTOMER.getVal());
        secretSetting.setSecretName(SecretTypeEnum.CONTACT_CUSTOMER.getName());
        secretSetting.setWxEnterpriseId(wxEnterpriseId);
        secretSetting.setEnterpriseId("-1");
        if (res != null) {
            secretSetting.setSecretVal(wxSecretKey);
            secretSetting.setCheckFlag(1);
            wxEnterpriseApiService.saveSecretSetting(secretSetting);
            return resultResponse(HaoBanErrCode.ERR_1, true);
        } else {
            secretSetting.setCheckFlag(0);
            wxEnterpriseApiService.saveSecretSetting(secretSetting);
            return resultResponse(HaoBanErrCode.ERR_10013);
        }
    }

    //微信企业会员小程序
    @RequestMapping("wx-enterprise-member-secret-set")
    public HaobanResponse wxEnterpriseMemberSecretSet(String memberSecret, String enterpriseId, String secretName) {
        if (StringUtils.isAnyBlank(memberSecret)) {
            return resultResponse(HaoBanErrCode.ERR_2);
        }
        String wxEnterpriseId = "";
        LoginDTO login = (LoginDTO) AuthRequestUtil.getLoginUser();
        wxEnterpriseId = login.getWxEnterpriseId();
        WxEnterpriseDTO wxEnterprise = wxEnterpriseApiService.getOne(wxEnterpriseId);
        if (wxEnterprise == null) {
            return resultResponse(HaoBanErrCode.ERR_10012);
        }
        memberSecret = memberSecret.trim();

        SecretSettingDTO secretSetting = wxEnterpriseApiService.getSecretSetting(wxEnterpriseId, SecretTypeEnum.MEMBER_WAPP.getVal(), enterpriseId);
        if (null == secretSetting) {
            secretSetting = new SecretSettingDTO();
        } else {
            if (memberSecret.equals(secretSetting.getSecretVal())) {
                logger.info("secret已经保存过");
                return resultResponse(HaoBanErrCode.ERR_100017);
            }
        }
        UserDTO user = qywxUserApiService.getSelfWorkWxUser(wxEnterprise.getCorpid(), memberSecret, getWxUserId(wxEnterpriseId, login.getStaffDTO()));
        secretSetting.setSecretType(SecretTypeEnum.MEMBER_WAPP.getVal());
        secretSetting.setSecretName(secretName);
        secretSetting.setEnterpriseId(enterpriseId);
        secretSetting.setWxEnterpriseId(wxEnterpriseId);

        if (user != null) {
            secretSetting.setSecretVal(memberSecret);
            secretSetting.setCheckFlag(1);
            wxEnterpriseApiService.saveSecretSetting(secretSetting);
            return resultResponse(HaoBanErrCode.ERR_1, true);
        } else {
            secretSetting.setCheckFlag(0);
            wxEnterpriseApiService.saveSecretSetting(secretSetting);
            return resultResponse(HaoBanErrCode.ERR_10013);
        }
    }

    private String getWxUserId(String wxEnterpriseId, StaffDTO loginStaffDTO) {
        if (loginStaffDTO != null && (!StringUtils.isEmpty(loginStaffDTO.getWxUserId()))) {
            return loginStaffDTO.getWxUserId();
        }
        StaffDTO staffOne = staffApiService.getWxStaffOne(wxEnterpriseId);
        return staffOne.getWxUserId();
    }

    //微信企业绑定接口绑定通讯录key(好办助手)
    @RequestMapping("wx-enterprise-haoban-secret-set")
    public HaobanResponse wxEnterpriseHaobanSecretSet(String contactSecret, String memberSecret) {
        if (StringUtils.isNotBlank(memberSecret)) {
            contactSecret = memberSecret;
        }
        if (StringUtils.isAnyBlank(contactSecret)) {
            return resultResponse(HaoBanErrCode.ERR_2);
        }
        String wxEnterpriseId = "";
        LoginDTO login = (LoginDTO) AuthRequestUtil.getLoginUser();
        wxEnterpriseId = login.getWxEnterpriseId();
        WxEnterpriseDTO wxEnterprise = wxEnterpriseApiService.getOne(wxEnterpriseId);
        if (wxEnterprise == null) {
            return resultResponse(HaoBanErrCode.ERR_10012);
        }
        contactSecret = contactSecret.trim();

        UserDTO user = qywxUserApiService.getSelfWorkWxUser(wxEnterprise.getCorpid(), contactSecret, getWxUserId(wxEnterpriseId, login.getStaffDTO()));
        logger.info("【保存联系人key】user={}", JSON.toJSONString(user));
        SecretSettingDTO secretSetting = wxEnterpriseApiService.getSecretSetting(wxEnterpriseId, SecretTypeEnum.HAOBAN_HELP.getVal(), null);
        if (null == secretSetting) {
            secretSetting = new SecretSettingDTO();
        }
        secretSetting.setSecretType(SecretTypeEnum.HAOBAN_HELP.getVal());
        secretSetting.setSecretName(SecretTypeEnum.HAOBAN_HELP.getName());
        secretSetting.setWxEnterpriseId(wxEnterpriseId);
        secretSetting.setEnterpriseId("-1");
        if (user != null) {
            secretSetting.setSecretVal(contactSecret);
            secretSetting.setCheckFlag(1);
            wxEnterpriseApiService.saveSecretSetting(secretSetting);
            return resultResponse(HaoBanErrCode.ERR_1, true);
        } else {
            secretSetting.setCheckFlag(0);
            wxEnterpriseApiService.saveSecretSetting(secretSetting);
            return resultResponse(HaoBanErrCode.ERR_10013);
        }

    }


    //微信企业会员小程序
    @RequestMapping("secret-list")
    public HaobanResponse secretList() {
        String wxEnterpriseId = "";
        LoginDTO login = (LoginDTO) AuthRequestUtil.getLoginUser();
        wxEnterpriseId = login.getWxEnterpriseId();
        WxEnterpriseDTO wxEnterprise = wxEnterpriseApiService.getOne(wxEnterpriseId);
        if (wxEnterprise == null) {
            return resultResponse(HaoBanErrCode.ERR_10012);
        }

        List<SecretSettingDTO> listSecret = wxEnterpriseApiService.listSecret(wxEnterpriseId);
        List<SecretSettingVO> listRet = EntityUtil.changeEntityListByJSON(SecretSettingVO.class, listSecret);
        if (CollectionUtils.isNotEmpty(listRet)) {
            listRet.forEach(secretSettingVO -> {
                if ((!StringUtils.isEmpty(secretSettingVO.getEnterpriseId())) && (!secretSettingVO.getEnterpriseId().equals("-1"))) {
                    EnterpriseDTO enterpriseDTO = enterpriseService.getEnterpriseById(secretSettingVO.getEnterpriseId());
                    secretSettingVO.setEnterpriseName(enterpriseDTO == null ? "" : enterpriseDTO.getBrandName());
                }
            });
        }
        return resultResponse(HaoBanErrCode.ERR_1, listRet);
    }


    //微信企业会员小程序
    @RequestMapping("member-secret-del")
    public HaobanResponse delMemberSecret(String secretId) {
        if (StringUtils.isAnyBlank(secretId)) {
            return resultResponse(HaoBanErrCode.ERR_2);
        }
        boolean b = wxEnterpriseApiService.delSecretSetting(secretId);
        return resultResponse(HaoBanErrCode.ERR_1, b);
    }

    //企业信息
    @RequestMapping("enterprise-is-over")
    public HaobanResponse enterpriseIsOver(String enterpriseId) {
        if (StringUtils.isEmpty(enterpriseId)) {
            return resultResponse(HaoBanErrCode.ERR_2);
        }
        EnterpriseDTO enterprise = enterpriseService.getEnterpriseById(enterpriseId);
        logger.info("enterprise={}", JSON.toJSONString(enterprise));
        if (enterprise == null || enterprise.getExpireTime() == null || enterprise.getEnabledState() == null) {
            return resultResponse(HaoBanErrCode.ERR_1, false);
        }
        if (enterprise.getExpireTime().getTime() > System.currentTimeMillis() && enterprise.getEnabledState() == 1) {
            return resultResponse(HaoBanErrCode.ERR_1, false);
        }
        logger.info("【返回类型】overFlag={}", true);
        return resultResponse(HaoBanErrCode.ERR_1, true);
    }

    /**
     * 刷新企业微信好友
     *
     * @return
     */
    @RequestMapping("/fresh-wx-friend")
    public HaobanResponse freshWxFriend() {
        LoginDTO login = (LoginDTO) AuthRequestUtil.getLoginUser();
        String wxEnterpriseId = login.getWxEnterpriseId();
        String dayKey = "haoban_fresh-wx-friend-day" + wxEnterpriseId + DateUtil.dateToStr(new Date(), "yyyy-MM-dd");
        String monthKey = "haoban_fresh-wx-friend-month" + wxEnterpriseId + DateUtil.dateToStr(new Date(), "yyyy-MM");
        String dateKey = "haoban_fresh-wx-friend-date" + wxEnterpriseId;
        Integer dayCount = RedisUtil.getCache(dayKey) == null ? 0 : Integer.parseInt(RedisUtil.getCache(dayKey).toString());
        Integer monthCount = RedisUtil.getCache(monthKey) == null ? 0 : Integer.parseInt(RedisUtil.getCache(monthKey).toString());
        if (dayCount >= 1) {
            return resultResponse(HaoBanErrCode.ERR_100019);
        }
        if (monthCount >= 3) {
            return resultResponse(HaoBanErrCode.ERR_100020);
        }
        String taskId = dealSyncOperationApiService.createWxFriendTask(wxEnterpriseId, "刷新企微好友", login.getStaffDTO().getStaffId(), "刷新企微好友");
        if (StringUtils.isEmpty(taskId)) {
            return resultResponse(HaoBanErrCode.ERR_100021);
        }
        RedisUtil.setCache(dayKey, dayCount + 1);
        RedisUtil.setCache(monthKey, monthCount + 1);
        RedisUtil.setCache(dateKey, DateUtil.dateToStr(new Date(), DateUtil.FORMAT_DATETIME_19));
        return resultResponse(HaoBanErrCode.ERR_1);
    }

    private QuartzTaskDTO buildQuartzTaskDTO(String id) {
        QuartzTaskDTO taskDTO = new QuartzTaskDTO();
        taskDTO.setReferId(id);
        taskDTO.setTaskMethod("quartWxFriendClerk");
        taskDTO.setTaskService("com.gic.haoban.manage.api.service.DealSyncOperationApiService");
        return taskDTO;
    }


    /**
     * 获取企业微信好友刷新时间
     *
     * @return
     */
    @RequestMapping("/get-fresh-detail")
    public HaobanResponse getFreshDetail() {
        LoginDTO login = (LoginDTO) AuthRequestUtil.getLoginUser();
        String wxEnterpriseId = login.getWxEnterpriseId();
        String dateKey = "haoban_fresh-wx-friend-date" + wxEnterpriseId;
        RedisUtil.getCache(dateKey);
        return resultResponse(HaoBanErrCode.ERR_1, RedisUtil.getCache(dateKey));
    }

    /**
     * 微信企业绑定详情
     *
     * @param enterpriseId
     * @return
     */
    @RequestMapping("wx-enterprise-bind-detail")
    public HaobanResponse wxEnterpriseBindDetail(String enterpriseId) {
        if (StringUtils.isAnyBlank(enterpriseId)) {
            return resultResponse(HaoBanErrCode.ERR_2);
        }
        LoginDTO login = (LoginDTO) AuthRequestUtil.getLoginUser();
        String wxEnterpriseId = login.getWxEnterpriseId();

        WxEnterpriseRelationDetailDTO bindInfo = wxEnterpriseRelatedApiService.getEnterpriseBindInfo(wxEnterpriseId, enterpriseId);
        return resultResponse(HaoBanErrCode.ERR_1, bindInfo);

    }

    /**
     * 绑定门店列表
     *
     * @param enterpriseId
     * @return
     */
    @RequestMapping("bind-store-list")
    public HaobanResponse bindStoreList(String enterpriseId, String search, BasePageInfo basePageInfo) {
        LoginDTO login = (LoginDTO) AuthRequestUtil.getLoginUser();
        String wxEnterpriseId = login.getWxEnterpriseId();
        Page<BindStoreInfoDTO> retPage = wxEnterpriseRelatedApiService.pageBindStoreByEnterpriseId(wxEnterpriseId, enterpriseId, search, basePageInfo);
        return resultResponse(HaoBanErrCode.ERR_1, retPage);
    }

    /**
     * 绑定门店列表
     *
     * @param enterpriseId
     * @return
     */
    @RequestMapping("flush-bind-store-list")
    public HaobanResponse flushBindStoreList(String enterpriseId) {
        LoginDTO login = (LoginDTO) AuthRequestUtil.getLoginUser();
        String wxEnterpriseId = login.getWxEnterpriseId();
        ServiceResponse response = wxEnterpriseRelatedApiService.flushBindStoreByEnterpriseId(enterpriseId, wxEnterpriseId, login.getStaffDTO().getStaffId(), ChannelCodeEnum.SYNC_UNBIND.getCode());
        logger.info("返回信息：{}", JSONObject.toJSONString(response));
        if (response.getCode() != 1) {
            HaoBanErrCode.ERR_DEFINE.setMsg(response.getMessage());
            return resultResponse(HaoBanErrCode.ERR_DEFINE);
        }
        return resultResponse(HaoBanErrCode.ERR_1);
    }

    /**
     * 绑定小程序配置列表
     *
     * @return
     */
    @RequestMapping("list-miniprogram-setting")
    public HaobanResponse miniprogramList() {
        LoginDTO login = (LoginDTO) AuthRequestUtil.getLoginUser();
        String wxEnterpriseId = login.getWxEnterpriseId();
        List<MiniprogramSettingDTO> settingDTOS = wxEnterpriseApiService.listMiniprogramSetting(wxEnterpriseId);
        return resultResponse(HaoBanErrCode.ERR_1, settingDTOS);
    }

    /**
     * 保存小程序配置列表
     *
     * @return
     */
    @RequestMapping("save-miniprogram-setting")
    public HaobanResponse saveMiniprogram(MiniprogramSettingQo settingDTO) {
        if (StringUtils.isAnyBlank(settingDTO.getAppId(),
                settingDTO.getImageUrl(), settingDTO.getMiniprogramName(), settingDTO.getTitle())) {
            return resultResponse(HaoBanErrCode.ERR_2);
        }
        LoginDTO login = (LoginDTO) AuthRequestUtil.getLoginUser();
        String wxEnterpriseId = login.getWxEnterpriseId();
        settingDTO.setWxEnterpriseId(wxEnterpriseId);
        MiniprogramSettingDTO miniprogramSettingDTO = wxEnterpriseApiService.getMiniprogramSettingByAppId(wxEnterpriseId, settingDTO.getAppId());
        if (null != miniprogramSettingDTO
                && (!miniprogramSettingDTO.getMiniprogramSettingId().equals(settingDTO.getMiniprogramSettingId()))) {
            return resultResponse(HaoBanErrCode.ERR_100031);
        }
        wxEnterpriseApiService.saveMiniprogramSetting(EntityUtil.changeEntityByOrika(MiniprogramSettingDTO.class, settingDTO));
        return resultResponse(HaoBanErrCode.ERR_1);
    }

    /**
     * 获取小程序配置
     *
     * @return
     */
    @RequestMapping("get-miniprogram-setting")
    public HaobanResponse getMiniprogram(String enterpriseId) {
        if (StringUtils.isEmpty(enterpriseId)) {
            return resultResponse(HaoBanErrCode.ERR_2);
        }
        LoginDTO login = (LoginDTO) AuthRequestUtil.getLoginUser();
        String wxEnterpriseId = login.getWxEnterpriseId();
        MiniprogramSettingDTO settingDTO = wxEnterpriseApiService.getMiniprogramSettingByEid(wxEnterpriseId, enterpriseId);
        return resultResponse(HaoBanErrCode.ERR_1, settingDTO);
    }

    /**
     * 删除小程序配置
     *
     * @return
     */
    @RequestMapping("del-miniprogram-setting")
    public HaobanResponse delMiniprogram(String id) {
        if (StringUtils.isEmpty(id)) {
            return resultResponse(HaoBanErrCode.ERR_2);
        }
        wxEnterpriseApiService.delMiniprogramSettingByEid(id);
        return resultResponse(HaoBanErrCode.ERR_1);
    }

    /**
     * 分析素材
     *
     * @return
     */
    @RequestMapping("/agent-wx-enterprise-list")
    public HaobanResponse materialShare(String materialId) {
        LoginDTO loginUser = this.getLoginUser();
        String wxEnterpriseId = loginUser.getWxEnterpriseId();
        List<EnterpriseDetailDTO> detailDTOS = wxEnterpriseRelatedApiService.listEnterpriseByWxEnterpriseId(wxEnterpriseId);
        List<String> enterpriseIds = detailDTOS.stream().filter(dto -> dto.getWxEnterpriseType() == 0).map(dto -> dto.getEnterpriseId()).collect(Collectors.toList());

        List<EnterpriseDetailDTO> retList = wxEnterpriseRelatedApiService.listEnterpriseByEnterpriseIds(enterpriseIds);
        Map<String, EnterpriseDetailDTO> detailDTOMap = retList.stream().filter(dto -> dto.getWxEnterpriseType() == 1).collect(Collectors.toMap(dto -> dto.getWxEnterpriseId(), dto -> dto, (ne, ol) -> ol));
        Collection<EnterpriseDetailDTO> values = detailDTOMap.values();
        List<AgentEnterpriseDTO> enterpriseList = EntityUtil.changeEntityListByOrika(AgentEnterpriseDTO.class, values);

        if (CollectionUtils.isNotEmpty(values)) {
            List<MaterialDTO> materialDTOS = materialApiService.listByFromMaterialId(materialId);
            List<String> wxEids = new ArrayList<>();
            if (CollectionUtils.isNotEmpty(materialDTOS)) {
                wxEids = materialDTOS.stream().map(dto -> dto.getWxEnterpriseId()).collect(Collectors.toList());
            }

            List<String> finalWxEids = wxEids;
            enterpriseList.forEach(dto -> {
                WxEnterpriseDTO wxEnterpriseDTO = wxEnterpriseApiService.getOne(dto.getWxEnterpriseId());
                dto.setEnterpriseName(wxEnterpriseDTO.getCorpName());
                if (finalWxEids.contains(dto.getWxEnterpriseId())) {
                    dto.setBindFlag(1);
                } else {
                    dto.setBindFlag(0);
                }
            });
        }
        return resultResponse(HaoBanErrCode.ERR_1, enterpriseList);
    }

    /**
     * 保存欢迎语
     *
     * @return
     */
    @RequestMapping("save-welcome")
    public HaobanResponse savewelcome(String title, String welcomeContent, String welcomeImg) {
        LoginDTO loginUser = this.getLoginUser();
        String wxEnterpriseId = loginUser.getWxEnterpriseId();
        if (StringUtils.isAnyBlank(title, welcomeContent)) {
            return resultResponse(HaoBanErrCode.ERR_2);
        }
        WelcomeDTO welcomeDTO = new WelcomeDTO();
        welcomeDTO.setTitle(title);
        welcomeDTO.setWelcomeContent(welcomeContent);
        welcomeDTO.setWelcomeImg(welcomeImg);
        welcomeDTO.setWxEnterpriseId(wxEnterpriseId);
        wxEnterpriseApiService.saveWelcome(welcomeDTO);
        return resultResponse(HaoBanErrCode.ERR_1);
    }

    /**
     * 欢迎语列表
     *
     * @return
     */
    @RequestMapping("get-welcome")
    public HaobanResponse getWelcome() {
        LoginDTO loginUser = this.getLoginUser();
        String wxEnterpriseId = loginUser.getWxEnterpriseId();
        WelcomeDTO welcome = wxEnterpriseApiService.getWelcome(wxEnterpriseId);
        return resultResponse(HaoBanErrCode.ERR_1, welcome);
    }

    /**
     * 删除欢迎语
     *
     * @return
     */
    @RequestMapping("del-welcome")
    public HaobanResponse delWelcome() {
        LoginDTO loginUser = this.getLoginUser();
        String wxEnterpriseId = loginUser.getWxEnterpriseId();
        wxEnterpriseApiService.delWelcome(wxEnterpriseId);
        return resultResponse(HaoBanErrCode.ERR_1);
    }

    /**
     * 欢迎语开启关闭
     *
     * @return
     */
    @RequestMapping("set-welcome")
    public HaobanResponse setWelcome(int openFlag) {
        LoginDTO loginUser = this.getLoginUser();
        String wxEnterpriseId = loginUser.getWxEnterpriseId();
        wxEnterpriseApiService.setWelcome(wxEnterpriseId, openFlag);
        return resultResponse(HaoBanErrCode.ERR_1);
    }

    /**
     * 授权绑定-设置导购编辑权限
     *
     * @return
     */
    @IgnoreLogin
    @RequestMapping("set-clerk-edit")
    public HaobanResponse setClerkEdit(String wxEnterprieseRelatedId, int clerkEditFlag) {
        if (StringUtils.isBlank(wxEnterprieseRelatedId)) {
            logger.info("缺少参数");
            return resultResponse(HaoBanErrCode.ERR_2);
        }
        boolean b = wxEnterpriseRelatedApiService.setClerkEditFlag(wxEnterprieseRelatedId, clerkEditFlag);
        if (b) {
            return resultResponse(HaoBanErrCode.ERR_1);
        } else {
            return resultResponse(HaoBanErrCode.ERR_100032);
        }
    }

    /**
     * 是否同意授权
     *
     * @param openFlag
     * @return
     */
    @RequestMapping("agree_external_userid_flag")
    public HaobanResponse agreeExternalUseridFlag(Integer openFlag) {
        LoginDTO loginUser = this.getLoginUser();
        String wxEnterpriseId = loginUser.getWxEnterpriseId();
        wxEnterpriseApiService.agreeExternalUseridFlag(wxEnterpriseId, openFlag);
        return resultResponse(HaoBanErrCode.ERR_1);
    }

}
