package com.gic.haoban.manage.web.utils;

import com.auth0.jwt.JWT;
import com.auth0.jwt.JWTCreator;
import com.auth0.jwt.JWTVerifier;
import com.auth0.jwt.algorithms.Algorithm;
import com.auth0.jwt.interfaces.Claim;
import com.auth0.jwt.interfaces.DecodedJWT;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.HashMap;
import java.util.Map;

/**
 * 生成token
 *
 * @author mozhu
 * @date 2021-12-10 14:53:34
 */
public class JwtUtil {

    private static final Logger logger = LoggerFactory.getLogger(JwtUtil.class);


    /**
     * 密钥 base64加密过的
     */
    public static final String SECRET_KEY = "Wb9^vZBjaYiFyhj@dDv!2FXSeEWxJauW";
    /**
     * 发行人
     */
    public static final String ISSUER = "gic_haoban";


    /**
     * 生成token
     *
     * @param claims
     * @return
     */
    public static String genToken(Map<String, String> claims) {
        //使用HMAC256进行加密
        Algorithm algorithm = Algorithm.HMAC256(SECRET_KEY);
        //创建jwt
        JWTCreator.Builder builder = JWT.create().withIssuer(ISSUER);
        //传入参数
        claims.forEach(builder::withClaim);
        //签名加密
        return builder.sign(algorithm);
    }

    /**
     * 解密token
     *
     * @param token
     * @return
     * @throws RuntimeException
     */
    public static Map<String, String> parseToken(String token) {
        Algorithm algorithm = null;
        try {
            //使用HMAC256进行加密
            algorithm = Algorithm.HMAC256(SECRET_KEY);
        } catch (IllegalArgumentException e) {
            logger.error("解密jwt失败:{}", e.getMessage(), e);
            return null;
        }
        //解密
        JWTVerifier verifier = JWT.require(algorithm).withIssuer(ISSUER).build();
        DecodedJWT jwt = verifier.verify(token);
        Map<String, Claim> map = jwt.getClaims();
        Map<String, String> resultMap = new HashMap<>();
        map.forEach((k, v) -> resultMap.put(k, v.asString()));
        return resultMap;
    }
}
